/* Tara Graeve
 * Assignment 5
 */
import java.awt.geom.Point2D;
import java.util.ArrayList;


import edu.princeton.cs.introcs.StdDraw;

public class SpatialTree {
	private SpatialTreeNode root; 
	private int size; 

	public SpatialTree() {
		size = 0; 
	}

	public boolean add(Point2D point) {
		SpatialTreeNode insert = new SpatialTreeNode(point); 

		if(size == 0) {
			root = insert; 
		} 
		
		else { 
			SpatialTreeNode temp = search(root,point); 
			if(temp.getValue().equals(point)) {
				return false; // false if it's not in the tree means it already has been added 
			}

			if(temp.isXNode()) {
				if (point.getX() > temp.getValue().getX()) {
					temp.setRight(insert);
				}
				temp.setX();
			} else {
				if(point.getY() > temp.getValue().getY()) {
					temp.setRight(insert);
				} else {
					temp.setLeft(insert);
				}
				temp.setX(); 

			}
		}
		size++; 
		return true; 

	}
	

	public SpatialTreeNode search(SpatialTreeNode node, Point2D val) {
		if(node.getValue().equals(val)) {
			return node; 
		}

		if(node.isXNode()) { 
			if(node.getValue().getX() <= val.getX()) {
				if (!node.hasRight()) return node; 
				return search(node.getRight(),val); 
			} else {
				if(!node.hasLeft()) return node; 
				return search(node.getLeft(),val);
			}
		}

		else {
			if(node.getValue().getY() <= val.getY()) {
				if (!node.hasRight()) return node; 
				return search(node.getRight(),val); 
			} else {
				if(!node.hasLeft()) return node; 
				return search(node.getLeft(),val);
			}
		}

	}

	public void draw() {
		drawHelper(root); 

	}

	private void drawHelper(SpatialTreeNode node) {

		StdDraw.setPenColor(StdDraw.BLACK);
		StdDraw.setPenRadius(0.01);
		if(node != null) {
			Point2D point = node.getValue(); 
			StdDraw.point(point.getX(), point.getY());
			StdDraw.setPenRadius(); 

			if (node.isXNode()) {
				StdDraw.setPenColor(StdDraw.BOOK_RED); 
				StdDraw.line(point.getX(),0, point.getX(), 100);
			} else {
				StdDraw.setPenColor(StdDraw.BOOK_BLUE); 
				StdDraw.line(0, point.getY(), 100, point.getY());

			}
			drawHelper(node.getLeft());
			drawHelper(node.getRight()); 

		}

	}


	public ArrayList<Point2D> query(Point2D center, double radius) {
		ArrayList<Point2D> points = new ArrayList<>(); 
		queryHelper(points, root, center, radius); 
		
		return points; 
	}

	private void queryHelper(ArrayList<Point2D> points, SpatialTreeNode node, Point2D center, double radius) {
		// return a list of points inside the query circle
		
		if(node != null) {
			if(node.getValue().distance(center) < radius) { // check if inside circle
				points.add(node.getValue()); 
				StdDraw.setPenColor(StdDraw.YELLOW); 
				StdDraw.point(node.getValue().getX(), node.getValue().getY());
			}

			// recursively call until all necessary points covered and check positioning
			if(node.isXNode()) {
				if((center.getX() + radius) < node.getValue().getX()) {
					queryHelper(points,node.getLeft(),center,radius); 
				}
				else if((center.getX() - radius) > node.getValue().getX()) { 
					queryHelper(points,node.getRight(),center,radius); 
				} else {
					queryHelper(points,node.getRight(),center,radius); 
					queryHelper(points,node.getLeft(),center,radius); 
				}
			}
			else { // if it's not an x, check Ys 
				if((center.getY() + radius) < node.getValue().getY()) { 
					queryHelper(points,node.getLeft(),center,radius); 
				}
				else if((center.getY() - radius) > node.getValue().getY()) {  
					queryHelper(points,node.getRight(),center,radius); 
				} else {
					queryHelper(points,node.getRight(),center,radius); 
					queryHelper(points,node.getLeft(),center,radius); 
				}
			}

		}
	}

}
